from functools import wraps

from flask_restful import abort
from marshmallow import ValidationError
from sqlalchemy.exc import SQLAlchemyError

from portal.service.errors import PortalAPIError


def error_handler(func):
    """Wraps the method and catches an PortalAPIException and calls abort
    Args:
        func: Wrapped method

    Returns: Wrapper function

    """
    @wraps(func)
    def _func(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except PortalAPIError as ex:
            abort(ex.code, message=ex.message)
        except ValidationError as ex:
            abort(400, message=f"Validation failed on: {ex.messages}")
        except SQLAlchemyError as ex:
            abort(400, message=f"Database error: {ex}")

    return _func
