from datetime import timedelta

from portal.database.models import User, Course, Project, Group, Role, Submission, Review, ReviewItem
from portal.database import SubmissionState
from portal.tools import time


def init_data(app, db):
    with app.app_context():
        # users
        admin = User(uco=0, email="admin@page.com", username="admin")
        admin.name = "Admin Ultimate"
        admin.is_admin = True
        admin.set_password("789789")

        student1 = User(uco=111, email="student1@uni.cz", username="student1")
        student1.set_password('123456')
        student1.name = "Student One"

        student2 = User(uco=222, email="student2@uni.cz", username="student2")
        student2.set_password('123456')
        student2.name = "Student Two"

        teacher1 = User(uco=123, email="teacher1@uni.cz", username="teacher1")
        teacher1.set_password('123123')
        teacher1.name = "Teacher And Student"

        teacher2 = User(uco=234, email="teacher2@uni.cz", username="teacher2")
        teacher2.set_password('123123')
        teacher2.name = "Teacher Only"

        db.session.add_all([admin, student1, student2, teacher1, teacher2])
        db.session.commit()

        # courses
        cpp = Course(name="C++", codename="PB161")
        cpp.notes_access_token = "cpp_token"

        c = Course(name="C", codename="PB071")
        c.notes_access_token = "c_token"

        db.session.add_all([cpp, c])
        db.session.commit()

        # groups
        cpp_students = Group(course=cpp, name="cpp_seminar01")
        cpp_teachers = Group(course=cpp, name="cpp_teachers")

        c_students = Group(course=c, name="c_seminar01")
        c_teachers = Group(course=c, name="c_teachers")

        db.session.add_all([cpp_students, cpp_teachers, c_students, c_teachers])
        db.session.commit()

        # projects
        cpp_p1 = Project(course=cpp, name="HW01", test_files_source="repo1")
        cpp_p1.set_config(file_whitelist="main.cpp", pre_submit_script="python for kontr pre",
                          post_submit_script="python for kontr post", submission_scheduler_config="python for sub Q",
                          submission_parameters="{\"type\":\"text\"}", submissions_allowed_from=time.NOW,
                          submissions_allowed_to=time.NOW + timedelta(days=2), archive_from=time.NOW + timedelta(days=30))
        cpp_p2 = Project(course=cpp, name="HW02", test_files_source="repo2")
        cpp_p1.set_config(file_whitelist="main.cpp\nfunc.hpp\nfunc.cpp", pre_submit_script="python for kontr pre",
                          post_submit_script="python for kontr post", submission_scheduler_config="python for sub Q",
                          submission_parameters="{\"type\":\"text\"}",
                          submissions_allowed_from=time.NOW + timedelta(days=1),
                          submissions_allowed_to=time.NOW + timedelta(days=10),
                          archive_from=time.NOW + timedelta(days=30))

        c_p1 = Project(course=c, name="HW01", test_files_source="repo3")
        c_p1.set_config(file_whitelist="main.c", pre_submit_script="python for kontr pre",
                        post_submit_script="python for kontr post", submission_scheduler_config="python for sub Q",
                        submission_parameters="{\"type\":\"text\"}", submissions_allowed_from=time.NOW - timedelta(days=1),
                        submissions_allowed_to=time.NOW - timedelta(minutes=1), archive_from=time.NOW + timedelta(days=30))

        db.session.add_all([cpp_p1, cpp_p2, c_p1])
        db.session.commit()

        # roles
        cpp_student = Role(name='student', course=cpp)
        cpp_student.set_permissions(view_course_limited=True, read_roles=True, create_submissions=True, read_projects=True,
                                    read_submissions_own=True, read_reviews_own=True, write_reviews_own=True)
        cpp_student.description = "Student role"
        cpp_teacher = Role(name='teacher', course=cpp)
        cpp_teacher.set_permissions(view_course_full=True, update_course=True, handle_notes_access_token=True,
                                    assign_roles=True, write_roles=True, write_groups=True, read_groups=True,
                                    write_projects=True, read_projects=True, archive_projects=True,
                                    resubmit_submissions=True, evaluate_submissions=True, read_submissions_all=True,
                                    read_all_submission_files=True, read_reviews_all=True, write_reviews_all=True)
        cpp_teacher.description = "Teacher role"
        cpp_assistant = Role(name='assistant', course=cpp)
        cpp_assistant.description = "Assistant role"

        c_student = Role(name='student', course=c)
        c_student.set_permissions(view_course_limited=True, read_roles=True, create_submissions=True, read_projects=True,
                                  read_submissions_own=True, read_reviews_own=True, write_reviews_own=True)
        c_student.description = "Student role"
        c_teacher = Role(name='teacher', course=c)
        c_teacher.set_permissions(view_course_full=True, update_course=True, handle_notes_access_token=True,
                                  assign_roles=True, write_roles=True, write_groups=True, read_groups=True,
                                  write_projects=True, read_projects=True, archive_projects=True,
                                  resubmit_submissions=True, evaluate_submissions=True, read_submissions_all=True,
                                  read_all_submission_files=True, read_reviews_all=True, write_reviews_all=True)
        c_teacher.description = "Teacher role"

        db.session.add_all([cpp_assistant, cpp_student, cpp_teacher, c_student, c_teacher])
        db.session.commit()

        # relationships
        teacher_student = User.query.filter_by(username='teacher1').first()
        teacher_both = User.query.filter_by(username='teacher2').first()
        student_cpp = User.query.filter_by(username='student1').first()
        student_both = User.query.filter_by(username='student2').first()
        #   roles
        teacher_student.roles.append(cpp_teacher)
        teacher_student.roles.append(c_student)
        teacher_both.roles.append(cpp_teacher)
        teacher_both.roles.append(c_teacher)
        student_cpp.roles.append(cpp_student)
        student_both.roles.append(cpp_student)
        student_both.roles.append(c_student)
        #   groups
        teacher_student.groups.append(cpp_teachers)
        teacher_student.groups.append(c_students)
        teacher_both.groups.append(cpp_teachers)
        teacher_both.groups.append(c_teachers)
        student_cpp.groups.append(cpp_students)
        student_both.groups.append(cpp_students)
        student_both.groups.append(c_students)

        # submissions
        c_sub1 = Submission(user=teacher_student, project=c_p1, parameters="")
        c_sub1.state = SubmissionState.FINISHED
        c_sub2 = Submission(user=student_both, project=c_p1, parameters="")
        c_sub2.state = SubmissionState.ABORTED

        cpp_sub_p1_cancel = Submission(user=student_both, project=cpp_p1, parameters="")
        cpp_sub_p1_cancel.state = SubmissionState.CANCELLED
        cpp_sub_p1_abort = Submission(user=student_cpp, project=cpp_p1, parameters="")
        cpp_sub_p1_abort.state = SubmissionState.ABORTED
        cpp_sub_p1_finished = Submission(user=student_both, project=cpp_p1, parameters="")
        cpp_sub_p1_finished.state = SubmissionState.FINISHED
        cpp_sub_p1_in_progress = Submission(user=student_cpp, project=cpp_p1, parameters="")
        cpp_sub_p1_in_progress.state = SubmissionState.IN_PROGRESS

        db.session.add_all([c_sub1, c_sub2, cpp_sub_p1_cancel, cpp_sub_p1_abort, cpp_sub_p1_finished,
                            cpp_sub_p1_in_progress])
        db.session.commit()

        # reviews
        review = Review(c_sub1)
        ri1 = ReviewItem(user=teacher_both, review=review, file="main.c", line=1, content="problem here")
        ri2 = ReviewItem(user=teacher_both, review=review, file="main.c", line=5, content="good stuff")
        ri3 = ReviewItem(user=teacher_student, review=review, file="main.c", line=1, content="oops")

        db.session.add_all([review])
        db.session.commit()




